'''
Condition group where conditions are set
'''


class ConditionGroup():
    def __init__(self):
        self.conditions = []
        self.success = 'success'
        self.failed = 'failed'

    def add_condition(self, l1, optn, arg, then, else_value):
        input_json = {'l1': l1, 'optn': optn, 'arg': arg, 'then': then, 'else_value': else_value}
        self.conditions.append(input_json)

    @property
    def conditions(self):
        return self.conditions

    def response(self, success, failed):
        self.success = success
        self.failed = failed



import itertools

from pundit.base import PunditBase
from pundit.condition import ConditionGroup

__version__ = "0.0.9"

'''
Pundit class where structure is defined and
inherited features of pundit base and condition groups
'''


class Pundit(PunditBase, ConditionGroup):
    def __init__(self, pundit, conditions):
        self.pundit = pundit
        self.condition_val = conditions
        self.conditions = conditions.conditions
        self.response = []
        self.set_input = []

    def evaluate(self, arg1, arg2=None):
        arg_one = arg1.lower() if type(arg1) == str else arg1
        arg_two = arg2.lower() if type(arg2) == str else arg2

        # appending to a sample dump with the struture
        self.set_input = []
        self.response = []
        self.set_input.append({self.pundit.arg1: arg_one, self.pundit.arg2: arg_two})

        # checking for the conditions
        for yme in self.conditions:

            if yme['optn'] == 'IN':
                self.response.append(True) if True in [
                    yme['l1'] in self.set_input[0][yme['arg']]] else self.response.append(False)
            elif yme['optn'] == 'NOT_IN':
                self.response.append(False) if True in [
                    yme['l1'] in self.set_input[0][yme['arg']]] else self.response.append(True)
            elif yme['optn'] == 'NOT_IS':
                pass
            elif yme['optn'] == 'EQ':
                self.response.append(True) if True in [
                    yme['l1'] == self.set_input[0][yme['arg']]] else self.response.append(False)
            elif yme['optn'] == 'NOT_EQ':
                self.response.append(True) if True in [
                    yme['l1'] != self.set_input[0][yme['arg']]] else self.response.append(False)
            elif yme['optn'] == 'GT':
                self.response.append(True) if True in [
                    yme['l1'] < self.set_input[0][yme['arg']]] else self.response.append(False)
            elif yme['optn'] == 'LT':
                self.response.append(True) if True in [
                    yme['l1'] > self.set_input[0][yme['arg']]] else self.response.append(False)
            elif yme['optn'] == 'GTE':
                self.response.append(True) if True in [
                    yme['l1'] <= self.set_input[0][yme['arg']]] else self.response.append(False)
            elif yme['optn'] == 'LTE':
                self.response.append(True) if True in [
                    yme['l1'] >= self.set_input[0][yme['arg']]] else self.response.append(False)
            else:
                return "unsupported operation"

        return self.condition_val.success if True in self.response else self.condition_val.failed


class MathRuler():
    """Mathemaical rule operatons"""

    def __init__(self, for_this, then_that):
        self.condition = for_this;
        self.then = then_that

    def math(self, con):
        then_that = self.then
        for_this = self.condition
        exp_converter = then_that.replace(for_this, str(con))
        return eval(exp_converter)


class ListRuler():
    def __init__(self, arg_list):
        self.list = arg_list
        self.reduced = []
        for li in self.list:
            if type(li) == list:
                for cc in list(itertools.chain(li)):
                    self.reduced.append(cc)
            else:
                self.reduced.append(li)

    def fullfill(self, arg):
        return True if arg in self.reduced else False



import json


'''
PunditBase Parent Class
Arguments : name
'''

class PunditBase():


	def __init__(self):
		self.mine = 'asd'
	

	def input_preprocess(self, id, function):


		def lower(self, id):

			self.processed_input = []
			for x in self.input_set:
				if x['id'] == id:
					self.processed_input.append({'id': x['id'],'value': x['value'].lower(), 'type': x['type']})
				else:
					self.processed_input.append(x)
			return


		def upper(self, id):
			self.processed_input = []
			for x in self.input_set:
				if x['id'] == id:
					self.processed_input.append({{'id': x['id'],'value': x['value'].upper(), 'type': x['type']}})
				else:
					self.processed_input.append(x)
			return

		if function == 'lower':
			lower(self, id)
		elif function == 'upper':
			upper(self, id)
		else:
			pass


	def add_structure(self, arg1, arg2):
		self.structure = []
		self.arg1 = arg1
		self.arg2 = arg2
		self.structure.append({arg1:{}, arg2:{}})


	@property
	def structure(self):
	    return self.structure
	
	



from setuptools import setup
from pundit import __version__

setup(name='pundit',
      version=__version__,
      description="ITTT for python",
      url='https://github.com/sachinvettithanam/pundit',
      author=['Anoop Thomas Mathew','Sachin Philip Mathew'],
      author_email='me@imsach.in',
      keywords='rule engine',
      license='MIT',
      packages=['pundit'],
      zip_safe=False,
      )
